"""This module is for the translation mapping data of the
following document:

Document file: graphics_draw_circle.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Graphics draw_circle interface':
    '',

    'This page explains the `Graphics` class `draw_circle` method interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    '`draw_circle` interface draws the vector circle graphics.':
    '',

    '## Basic usage':
    '',

    'The `draw_circle` interface has the `x`\\, `y`\\, and `radius` arguments.\n\nThe `x` and `y` arguments are the circle center coordinates, and the radius argument determines the circle size. The width and height become twice the `radius` value.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=350,\n    stage_height=200,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\n\n# Set the cyan color and draw the circle.\nsprite.graphics.begin_fill(color=\'#0af\')\nsprite.graphics.draw_circle(x=100, y=100, radius=50)\n\n# Set the dotted-line style and draw the circle.\nsprite.graphics.begin_fill(color=\'\')\nsprite.graphics.line_style(\n    color=\'#fff\', thickness=3, dot_setting=ap.LineDotSetting(dot_size=3))\nsprite.graphics.draw_circle(x=250, y=100, radius=50)\n\n# Draw the inner circle.\nsprite.graphics.draw_circle(x=250, y=100, radius=25)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_draw_circle_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_draw_circle_basic_usage/index.html" width="350" height="200"></iframe>':  # noqa
    '',

    '## Return value':
    '',

    'The return value of the `draw_circle` interface is the instance of the `Circle` class.\n\nIt has the `radius` attribute or other basic interfaces and you can change these settings.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=400,\n    stage_height=400,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\n\n# Draw the small radius circle.\nsprite.graphics.begin_fill(color=\'#0af\')\ncircle: ap.Circle = sprite.graphics.draw_circle(x=200, y=200, radius=25)\n\n# Update circle radius to become the bigger one.\ncircle.radius = ap.Int(100)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_draw_circle_return_value/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_draw_circle_return_value/index.html" width="400" height="400"></iframe>':  # noqa
    '',

    '## draw_circle API':
    '',

    '<!-- Docstring: apysc._display.graphics.Graphics.draw_circle -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `draw_circle(self, x:Union[int, apysc._type.int.Int], y:Union[int, apysc._type.int.Int], radius:Union[int, apysc._type.int.Int]) -> \'_circle.Circle\'`<hr>\n\n**[Interface summary]** Draw a circle vector graphics.<hr>\n\n**[Parameters]**\n\n- `x`: Int or int\n  - X-coordinate of the circle center.\n- `y`: Int or int\n  - Y-coordinate of the circle center.\n- `radius`: Int or int\n  - Circle radius.\n\n<hr>\n\n**[Returns]**\n\n- `circle`: Circle\n  - Created circle graphics instance.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> circle: ap.Circle = sprite.graphics.draw_circle(\n...     x=100, y=100, radius=50)\n>>> circle.x\nInt(100)\n\n>>> circle.y\nInt(100)\n\n>>> circle.radius\nInt(50)\n\n>>> circle.fill_color\nString(\'#00aaff\')\n```':  # noqa
    '',

}



"""This module is the helper module for the flip interfaces.
"""

from enum import Enum

from apysc._type.boolean import Boolean


class Axis(Enum):
    X = 'x'
    Y = 'y'


def make_flip_update_expression(
        *, before_value: Boolean, after_value: Boolean,
        axis: Axis, interface_variable_name: str) -> str:
    """
    Make a flipping value updating expression.

    Parameters
    ----------
    before_value : Boolean
        Before updating flipping value.
    after_value : Boolean
        After updating flipping value.
    axis : Axis
        X or y axis value.
    interface_variable_name : str
        Interface instance variable name.

    Returns
    -------
    expression : str
        Made expression string.
    """
    from apysc._type import value_util
    before_value_str: str = value_util.get_value_str_for_expression(
        value=before_value)
    after_value_str: str = value_util.get_value_str_for_expression(
        value=after_value)
    expression: str = (
        f'if ({before_value_str}) {{'
        f'\n  {interface_variable_name}.flip("{axis.value}");'
        '\n}'
        f'\nif ({after_value_str}) {{'
        f'\n  {interface_variable_name}.flip("{axis.value}");'
        '\n}'
        f'\n{before_value_str} = {after_value_str};'
    )
    return expression



"""Class implementation for the timer event.
"""

from apysc._event.event import Event
from apysc._html.debug_mode import add_debug_info_setting
from apysc._time import timer


class TimerEvent(Event):
    """
    Timer event class.

    References
    ----------
    - TimerEvent class document
        - https://simon-ritchie.github.io/apysc/timer_event.html

    Examples
    --------
    >>> from typing_extensions import TypedDict
    >>> import apysc as ap
    >>> class RectOptions(TypedDict):
    ...     rectangle: ap.Rectangle
    >>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:
    ...     rectangle: ap.Rectangle = options['rectangle']
    ...     rectangle.x += 1
    ...     with ap.If(rectangle.x >= 100):
    ...         timer: ap.Timer = e.this
    ...         timer.stop()
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color='#0af')
    >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
    ...     x=50, y=50, width=50, height=50)
    >>> options: RectOptions = {'rectangle': rectangle}
    >>> ap.Timer(
    ...     on_timer, delay=ap.FPS.FPS_60, options=options,
    ... ).start()
    """

    _this: 'timer.Timer'

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='TimerEvent')
    def __init__(self, this: 'timer.Timer') -> None:
        """
        Timer event class.

        Parameters
        ----------
        this : Timer
            Target timer instance.

        References
        ----------
        - TimerEvent class document
            - https://simon-ritchie.github.io/apysc/timer_event.html

        Examples
        --------
        >>> from typing_extensions import TypedDict
        >>> import apysc as ap
        >>> class RectOptions(TypedDict):
        ...     rectangle: ap.Rectangle
        >>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:
        ...     rectangle: ap.Rectangle = options['rectangle']
        ...     rectangle.x += 1
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> options: RectOptions = {'rectangle': rectangle}
        >>> ap.Timer(
        ...     on_timer, delay=ap.FPS.FPS_60, options=options,
        ... ).start()
        """
        from apysc._expression import var_names
        super(TimerEvent, self).__init__(
            this=this, type_name=var_names.TIMER_EVENT)

    @property
    def this(self) -> 'timer.Timer':
        """
        Get a timer instance of listening to this event.

        Returns
        -------
        this : TImer
            Instance of listening to this event.

        References
        ----------
        - TimerEvent class document
            - https://simon-ritchie.github.io/apysc/timer_event.html

        Examples
        --------
        >>> from typing_extensions import TypedDict
        >>> import apysc as ap
        >>> class RectOptions(TypedDict):
        ...     rectangle: ap.Rectangle
        >>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:
        ...     rectangle: ap.Rectangle = options['rectangle']
        ...     rectangle.x += 1
        ...     with ap.If(rectangle.x >= 100):
        ...         timer: ap.Timer = e.this
        ...         timer.stop()
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> options: RectOptions = {'rectangle': rectangle}
        >>> ap.Timer(
        ...     on_timer, delay=ap.FPS.FPS_60, options=options,
        ... ).start()
        """
        return self._this



"""This module is for the translation mapping data of the
following document:

Document file: display_on_colaboratory.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# display_on_colaboratory interface':
    '',

    'This page will explain the `display_on_colaboratory` function interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    'The `display_on_colaboratory` interface displays the apysc HTML on the Google Colaboratory.':  # noqa
    '',

    '## Requirements':
    '',

    'You need to install apysc on the Google Colaboratory before going on. A `!` symbol and pip command on the Google Colaboratory installs this library:':  # noqa
    '',

    '```\n!pip install apysc\n```':
    '',

    '':
    '',

    '## Basic usage':
    '',

    'You can use the `display_on_colaboratory` interface to display an output HTML instead of the `save_overall_html` function.\n\nThis interface requires the `html_file_name` argument to be unique if you need to output multiple HTML. Otherwise, it overwrites the HTML file:':  # noqa
    '',

    '```py\nimport apysc as ap\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color=\'#333\')\nsprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nsprite.graphics.begin_fill(color=\'#f0a\')\nsprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\n\nap.display_on_colaboratory(html_file_name=\'jupyter_test_1.html\')\n```':  # noqa
    '',

    '![](_static/colaboratory_interface.png)':
    '',

    '## display_on_colaboratory API':
    '',

    '<!-- Docstring: apysc._jupyter.jupyter_util.display_on_colaboratory -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `display_on_colaboratory(html_file_name:str, *, minify:bool=True) -> None`<hr>\n\n**[Interface summary]** Save the overall HTML and display it on Google Colaboratory.<hr>\n\n**[Parameters]**\n\n- `html_file_name`: str, default \'index.html\'\n  - The output HTML file name.\n- `minify`: bool, default True\n  - Boolean value whether minify a HTML or not. False setting is useful when debugging.':  # noqa
    '',

}



"""Implementation of the Ellipse class.
"""

from typing import Union

from apysc._display import graphics
from apysc._display.cx_interface import CxInterface
from apysc._display.cy_interface import CyInterface
from apysc._display.line_base import LineBase
from apysc._display.width_and_height_interfaces_for_ellipse import \
    WidthAndHeightInterfacesForEllipse
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int


class Ellipse(  # type: ignore
        CxInterface, CyInterface, LineBase,
        WidthAndHeightInterfacesForEllipse):
    """
    The ellipse vector graphics class.

    References
    ----------
    - Graphics draw_ellipse interface
        - https://simon-ritchie.github.io/apysc/graphics_draw_ellipse.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color='#0af')
    >>> ellipse: ap.Ellipse = sprite.graphics.draw_ellipse(
    ...     x=100, y=100, width=80, height=50)
    >>> ellipse.x
    Int(100)

    >>> ellipse.y
    Int(100)

    >>> ellipse.width
    Int(80)

    >>> ellipse.height
    Int(50)

    >>> ellipse.fill_color
    String('#00aaff')
    """

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Ellipse')
    def __init__(
            self,
            *,
            parent: 'graphics.Graphics',
            x: Union[int, Int],
            y: Union[int, Int],
            width: Union[int, Int],
            height: Union[int, Int]) -> None:
        """
        Create an ellipse vector graphic.

        Parameters
        ----------
        parent : Graphics
            Graphics instance to link this graphic.
        x : Int or int
            X-coordinate of the ellipse center.
        y : Int or int
            Y-coordinate of the ellipse center.
        width : Int or int
            Ellipse width.
        height : Int or int
            Ellipse height.

        References
        ----------
        - Graphics draw_ellipse interface
            - https://simon-ritchie.github.io/apysc/graphics_draw_ellipse.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> ellipse: ap.Ellipse = sprite.graphics.draw_ellipse(
        ...     x=100, y=100, width=80, height=50)
        >>> ellipse.x
        Int(100)

        >>> ellipse.y
        Int(100)

        >>> ellipse.width
        Int(80)

        >>> ellipse.height
        Int(50)

        >>> ellipse.fill_color
        String('#00aaff')
        """
        from apysc._converter.to_apysc_val_from_builtin import \
            get_copied_int_from_builtin_val
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._validation import size_validation
        variable_name: str = expression_variables_util.\
            get_next_variable_name(type_name=var_names.ELLIPSE)
        super(Ellipse, self).__init__(
            parent=parent, x=0, y=0, variable_name=variable_name)
        size_validation.validate_size_is_gt_zero(size=width)
        size_validation.validate_size_is_gt_zero(size=height)
        self._width = get_copied_int_from_builtin_val(integer=width)
        self._height = get_copied_int_from_builtin_val(integer=height)
        self._set_initial_basic_values(parent=parent)
        self._append_constructor_expression()
        self.x = get_copied_int_from_builtin_val(integer=x)
        self.y = get_copied_int_from_builtin_val(integer=y)
        self._set_line_setting_if_not_none_value_exists(
            parent_graphics=parent)
        self._set_overflow_visible_setting()

        self._append_width_attr_linking_setting()
        self._append_height_attr_linking_setting()

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Ellipse')
    def _append_constructor_expression(self) -> None:
        """
        Append a constructor expression.
        """
        import apysc as ap
        from apysc._display.stage import get_stage_variable_name
        from apysc._type import value_util
        stage_variable_name: str = get_stage_variable_name()
        width_str: str = value_util.get_value_str_for_expression(
            value=self._width)
        height_str: str = value_util.get_value_str_for_expression(
            value=self._height)
        expression: str = (
            f'var {self.variable_name} = {stage_variable_name}'
            f'\n  .ellipse({width_str}, {height_str})'
            '\n  .attr({'
        )
        expression = self._append_basic_vals_expression(
            expression=expression, indent_num=2)
        expression += '\n  });'
        ap.append_js_expression(expression=expression)

    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).

        Returns
        -------
        repr_str : str
            Type name and variable name will be set
            (e.g., `Ellipse('<variable_name>')`).
        """
        repr_str: str = f"Ellipse('{self.variable_name}')"
        return repr_str



"""This module is for the translation mapping data of the
following document:

Document file: graphics_move_to_and_line_to.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Graphics move_to and line_to interfaces':
    '',

    'This page explains the `Graphics` class `move_to` and `line_to` method interfaces.':  # noqa
    '',

    '## What interfaces are they?':
    '',

    'The `move_to` interface sets the line start point. The `line_to` draws the line from a current point to a destination point. Sequentially, if you call the `line_to` interface, the line becomes polyline.\n\nIf you call the `move_to` interface after the calling of `line_to`\\, it creates a new line instance.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `move_to` and `line_to` interfaces have x and y arguments.':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=300,\n    stage_height=100,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.line_style(color=\'#0af\', thickness=5)\n\n# Move to x=50, y=50 point (no drawing).\nsprite.graphics.move_to(x=50, y=50)\n\n# Draw the line from the current point (50, 50) to the\n# destination point (250, 50).\nsprite.graphics.line_to(x=250, y=50)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_move_to_and_line_to_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_move_to_and_line_to_basic_usage/index.html" width="300" height="100"></iframe>':  # noqa
    '',

    '## Sequential calling of the line_to interface':
    '',

    'Sequentially, if you call the `line_to` interface, the result line becomes the polyline.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.line_style(color=\'#0af\', thickness=5)\n\n# Move to x=50, y=50 point (no drawing).\nsprite.graphics.move_to(x=50, y=50)\n\n# Draw the line from the current point (50, 50) to the\n# destination point (150, 50).\nsprite.graphics.line_to(x=150, y=50)\n\n# Draw the line from the current point (250, 50) to the\n# destination point (50, 150). This calling changes the line\n# to the polyline.\nsprite.graphics.line_to(x=50, y=150)\n\n# Finally the polyline becomes Z shape by drawing to\n# destination point (150, 150).\nsprite.graphics.line_to(x=150, y=150)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_move_to_and_line_to_sequential_calling/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_move_to_and_line_to_sequential_calling/index.html" width="200" height="200"></iframe>':  # noqa
    '',

    '## move_to interface calling after line_to interface calling':
    '',

    'If you call the `move_to` interface after calling the `line_to` interface, it creates a new line instance.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.line_style(color=\'#0af\', thickness=5)\n\n# First move_to interface calling.\nsprite.graphics.move_to(x=50, y=50)\nsprite.graphics.line_to(x=100, y=50)\nsprite.graphics.line_to(x=50, y=100)\nsprite.graphics.line_to(x=100, y=100)\n\n# Second move_to interface calling. This will create a new\n# polyline instance.\nsprite.graphics.move_to(x=150, y=50)\nsprite.graphics.line_to(x=200, y=50)\nsprite.graphics.line_to(x=150, y=100)\nsprite.graphics.line_to(x=200, y=100)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_move_to_and_line_to_multi_move_to_calling/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_move_to_and_line_to_multi_move_to_calling/index.html" width="250" height="150"></iframe>':  # noqa
    '',

    '## Polyline instance':
    '',

    '`move_to` and `line_to` interfaces will return `Polyline` instance. You can update each setting or bind events to that instance.\n\nFor instance, the following script sets the mouse event to `Polyline`\\, updates the line color, and sets dot style in the handler (`on_line_click`).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_line_click(\n        e: ap.MouseEvent[ap.Polyline], options: dict) -> None:\n    """\n    The handler that the line instance calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        The event instance.\n    options : dict\n        Optional arguments.\n    """\n    polyline: ap.Polyline = e.this\n    polyline.line_color = ap.String(\'#f0a\')\n    polyline.line_dot_setting = ap.LineDotSetting(dot_size=5)\n\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=100,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.line_style(color=\'#0af\', thickness=30)\npolyline: ap.Polyline = sprite.graphics.move_to(x=50, y=50)\nsprite.graphics.line_to(x=150, y=50)\npolyline.click(on_line_click)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_move_to_and_line_to_polyline/\')\n```':  # noqa
    '',

    'If you click the following line, line style will be updated:\n\n<iframe src="static/graphics_move_to_and_line_to_polyline/index.html" width="200" height="100"></iframe>':  # noqa
    '',

    '## move_to API':
    '',

    '<!-- Docstring: apysc._display.graphics.Graphics.move_to -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `move_to(self, x:Union[int, apysc._type.int.Int], y:Union[int, apysc._type.int.Int]) -> \'_polyline.Polyline\'`<hr>\n\n**[Interface summary]** Move a line position to a specified point.<hr>\n\n**[Parameters]**\n\n- `x`: Int or int\n  - X destination point to move.\n- `y`: Int or int\n  - Y destination point to move.\n\n<hr>\n\n**[Returns]**\n\n- `line`: Polyline\n  - Line graphics instance.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=\'#fff\', thickness=5)\n>>> line_1: ap.Polyline = sprite.graphics.move_to(x=50, y=50)\n>>> line_2: ap.Polyline = sprite.graphics.line_to(x=150, y=50)\n>>> line_1 == line_2\nTrue\n\n>>> line_1.line_color\nString(\'#ffffff\')\n\n>>> line_1.line_thickness\nInt(5)\n```':  # noqa
    '',

    '':
    '',

    '## line_to API':
    '',

    '<!-- Docstring: apysc._display.graphics.Graphics.line_to -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `line_to(self, x:Union[int, apysc._type.int.Int], y:Union[int, apysc._type.int.Int]) -> \'_polyline.Polyline\'`<hr>\n\n**[Interface summary]** Draw a line from previous point to specified point (initial point is x = 0, y = 0).<hr>\n\n**[Parameters]**\n\n- `x`: Int or int\n  - X destination point to draw a line.\n- `y`: Int or int\n  - Y destination point to draw a line.\n\n<hr>\n\n**[Returns]**\n\n- `line`: Polyline\n  - Line graphics instance.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=\'#fff\', thickness=5)\n>>> line_1: ap.Polyline = sprite.graphics.move_to(x=50, y=50)\n>>> line_2: ap.Polyline = sprite.graphics.line_to(x=150, y=50)\n>>> line_3: ap.Polyline = sprite.graphics.line_to(x=50, y=150)\n>>> line_1 == line_2 == line_3\nTrue\n\n>>> line_1.line_color\nString(\'#ffffff\')\n\n>>> line_1.line_thickness\nInt(5)\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: fundamental_data_classes_value_interface.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# apysc fundamental data classes value interface':
    '',

    'This page explains the apysc fundamental data classes (such as the `Int`\\, `Number`\\, `String`) `value` interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    'The `value` getter interface returns each data class value. And the setter interface updates these data class values.\n\nA return value of the getter interface becomes a Python built-in value, like the `int`\\, `float`\\, `str`\\, `list`\\.':  # noqa
    '',

    '## Basic usage of the getter interface':
    '',

    'The `value` getter interface returns the Python built-in value.':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nvalue = int_1.value\nassert isinstance(value, int)\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nnumber_1: ap.Number = ap.Number(10.5)\nvalue = number_1.value\nassert isinstance(value, float)\n```':  # noqa
    '',

    '':
    '',

    '## Basic usage of the setter interface':
    '',

    'You can update the apysc fundamental data class values with the `value` setter interface. Python built-in values and the same type value is acceptable:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1.value = 20\nassert int_1 == 20\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1.value = ap.Int(20)\nassert int_1 == 20\n```':  # noqa
    '',

}



"""Class implementation for begin_fill method-related interface.

See Also
--------
- graphics_clear_interface
"""


from typing import Dict
from typing import TypeVar
from typing import Union

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number
from apysc._type.revert_interface import RevertInterface
from apysc._type.string import String

StrOrString = TypeVar('StrOrString', str, String)


class BeginFillInterface(RevertInterface):

    _fill_color: String
    _fill_alpha: Number

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='BeginFillInterface')
    def begin_fill(
            self, color: StrOrString,
            *,
            alpha: Union[float, Number] = 1.0) -> None:
        """
        Set single color value for fill.

        Parameters
        ----------
        color : str or String
            Hexadecimal color string. e.g., '#00aaff'
        alpha : float or Number, default 1.0
            Color opacity (0.0 to 1.0).

        References
        ----------
        - Graphics begin_fill interface document
            - https://simon-ritchie.github.io/apysc/graphics_begin_fill.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> rectangle.fill_color
        String('#00aaff')
        """
        import apysc as ap
        from apysc._color import color_util
        from apysc._converter import cast
        from apysc._validation import color_validation
        from apysc._validation import number_validation
        self._initialize_fill_color_if_not_initialized()
        self._initialize_fill_alpha_if_not_initialized()
        if color != '':
            color = color_util.complement_hex_color(
                hex_color_code=color)
        self._fill_color.value = color
        number_validation.validate_num(num=alpha)
        if not isinstance(alpha, ap.Number):
            alpha = cast.to_float_from_int(int_or_float=alpha)
        color_validation.validate_alpha_range(alpha=alpha)
        if isinstance(alpha, ap.Number):
            self._fill_alpha.value = alpha.value
        else:
            self._fill_alpha.value = alpha

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='BeginFillInterface')
    def fill_color(self) -> String:
        """
        Get current fill color.

        Returns
        -------
        fill_color : String
            Current fill color (hexadecimal string, e.g., '#00aaff').
            If not be set, this interface returns a blank string.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> rectangle.fill_color
        String('#00aaff')
        """
        import apysc as ap
        from apysc._type import value_util
        self._initialize_fill_color_if_not_initialized()
        fill_color: ap.String = value_util.get_copy(value=self._fill_color)
        return fill_color

    def _initialize_fill_color_if_not_initialized(self) -> None:
        """
        Initialize the fill_color attribute if this interface
        does not initialize it yet.
        """
        if hasattr(self, '_fill_color'):
            return
        self._fill_color = String('')

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='BeginFillInterface')
    def fill_alpha(self) -> Number:
        """
        Get current fill color opacity.

        Returns
        -------
        fill_alpha : Number
            Current fill color opacity (0.0 to 1.0).
            If not be set, 1.0 will be returned.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af', alpha=0.5)
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> rectangle.fill_alpha
        Number(0.5)
        """
        import apysc as ap
        from apysc._type import value_util
        self._initialize_fill_alpha_if_not_initialized()
        fill_alpha: ap.Number = value_util.get_copy(value=self._fill_alpha)
        return fill_alpha

    def _initialize_fill_alpha_if_not_initialized(self) -> None:
        """
        Initialize the fill_alpha attribute if this interface
        does not initialize it yet.
        """
        if hasattr(self, '_fill_alpha'):
            return
        self._fill_alpha = Number(1.0)

    _fill_color_snapshots: Dict[str, str]
    _fill_alpha_snapshots: Dict[str, float]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make values' snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_fill_color_if_not_initialized()
        self._initialize_fill_alpha_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name='_fill_color_snapshots',
            value=self._fill_color._value, snapshot_name=snapshot_name)
        self._set_single_snapshot_val_to_dict(
            dict_name='_fill_alpha_snapshots',
            value=self._fill_alpha._value, snapshot_name=snapshot_name)

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert values if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._fill_color._value = self._fill_color_snapshots[snapshot_name]
        self._fill_alpha._value = self._fill_alpha_snapshots[snapshot_name]



"""This module is for the translation mapping data of the
following document:

Document file: graphics_begin_fill.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Graphics begin_fill interface':
    '',

    'This page explains the `Graphics` class `begin_fill` method interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    '`begin_fill` interface would set the fill color and fill alpha settings. This setting would be maintained until it is called again or called the `clear` method.':  # noqa
    '',

    '## Basic usage':
    '',

    'Draw vector graphics interfaces (e.g., `draw_rect`) would use these fill settings when creating, so the `begin_fill` method needs to be called before calling each drawing interface.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=350,\n    stage_height=150,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Set blue fill color and draw the first rectangle.\nsprite.graphics.begin_fill(color=\'#0af\')\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\n# Draw the second rectangle (fill color setting will be maintained).\nsprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50)\n\n# Set the other fill color and draw the third rectangle.\nsprite.graphics.begin_fill(color=\'#f0a\')\nsprite.graphics.draw_rect(\n    x=250, y=50, width=50, height=50)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_begin_fill_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_begin_fill_basic_usage/index.html" width="350" height="150"></iframe>':  # noqa
    '',

    '## Fill color setting':
    '',

    'The `color` argument sets the fill color, and the `begin_fill` interface requires this argument.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Set a cyan fill color and draw the rectangle.\nsprite.graphics.begin_fill(color=\'#0af\')\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_begin_fill_fill_color/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_begin_fill_fill_color/index.html" width="150" height="150"></iframe>\n\nIf you want to clear fill color, specify a blank string to this argument.\n\nFor example, since the following code clears fill color settings, a rectangle graphic becomes invisible.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=\'#0af\')\n\n# Clear fill color by specifying blank string.\nsprite.graphics.begin_fill(color=\'\')\n\n# Since fill color is not set, the rectangle is invisible.\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_begin_fill_color_setting_clear/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_begin_fill_color_setting_clear/index.html" width="150" height="150"></iframe>\n\nColor code is acceptable like the following list:\n\n- Six characters, e.g., `#00aaff`.\n- Three characters, e.g., `#0af` (this becomes `#00aaff`).\n- Single character, e.g., `#5` (this becomes `#000005`).\n- Skipped `#` symbol, e.g., `0af` (this becomes `#00aaff`).\n- Blank string, e.g., `\'\'` (this setting clears the fill color setting).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=450,\n    stage_height=150,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Six characters fill color setting (a cyan color).\nsprite.graphics.begin_fill(color=\'#00aaff\')\nsprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\n# Three characters fill color setting (a magenta color).\nsprite.graphics.begin_fill(color=\'#f0a\')\nsprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50)\n\n# Single characters fill color setting (a black color).\nsprite.graphics.begin_fill(color=\'#0\')\nsprite.graphics.draw_rect(\n    x=250, y=50, width=50, height=50)\n\n# Fill color that Skipped `#` symbol is also acceptable.\nsprite.graphics.begin_fill(color=\'999\')\nsprite.graphics.draw_rect(\n    x=350, y=50, width=50, height=50)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_begin_fill_acceptable_color_settings/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_begin_fill_acceptable_color_settings/index.html" width="450" height="150"></iframe>':  # noqa
    '',

    '## Fill color alpha (opacity) setting':
    '',

    'Fill color alpha (opacity) can be set by the `alpha` argument. It can accept 0.0 (transparent) to 1.0 (opaque).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=\'#00aaff\', alpha=0.2)\nsprite.graphics.draw_rect(\n    x=50, y=75, width=50, height=50)\nsprite.graphics.draw_rect(\n    x=75, y=50, width=50, height=50)\nsprite.graphics.draw_rect(\n    x=75, y=75, width=50, height=50)\nsprite.graphics.draw_rect(\n    x=75, y=100, width=50, height=50)\nsprite.graphics.draw_rect(\n    x=100, y=75, width=50, height=50)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_begin_fill_alpha_setting/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_begin_fill_alpha_setting/index.html" width="200" height="200"></iframe>':  # noqa
    '',

    '## begin_fill API':
    '',

    '<!-- Docstring: apysc._display.begin_fill_interface.BeginFillInterface.begin_fill -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `begin_fill(self, color:~StrOrString, *, alpha:Union[float, apysc._type.number.Number]=1.0) -> None`<hr>\n\n**[Interface summary]** Set single color value for fill.<hr>\n\n**[Parameters]**\n\n- `color`: str or String\n  - Hexadecimal color string. e.g., \'#00aaff\'\n- `alpha`: float or Number, default 1.0\n  - Color opacity (0.0 to 1.0).\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> rectangle.fill_color\nString(\'#00aaff\')\n```':  # noqa
    '',

    '':
    '',

    '## fill_color property API':
    '',

    '<!-- Docstring: apysc._display.begin_fill_interface.BeginFillInterface.fill_color -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get current fill color.<hr>\n\n**[Returns]**\n\n- `fill_color`: String\n  - Current fill color (hexadecimal string, e.g., \'#00aaff\'). If not be set, this interface returns a blank string.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> rectangle.fill_color\nString(\'#00aaff\')\n```':  # noqa
    '',

    '':
    '',

    '## fill_alpha property API':
    '',

    '<!-- Docstring: apysc._display.begin_fill_interface.BeginFillInterface.fill_alpha -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get current fill color opacity.<hr>\n\n**[Returns]**\n\n- `fill_alpha`: Number\n  - Current fill color opacity (0.0 to 1.0). If not be set, 1.0 will be returned.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\', alpha=0.5)\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> rectangle.fill_alpha\nNumber(0.5)\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: bind_and_trigger_custom_event.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Bind and trigger the custom event':
    '',

    'This page explains the `bind_custom_event` and `trigger_custom_event` interfaces.':  # noqa
    '',

    '## What interfaces are these?':
    '',

    'The `bind_custom_event` interface registers your custom event to the instance, and the `trigger_custom_event` one triggers the registered custom events at any time.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `bind_custom_event` interface has the `custom_event_type`, `handler`, `e`, and `options` arguments (`options` is optional).\n\nThe `custom_event_type` argument is the custom event type name\'s string. This value needs to specify the same one at the calling of the `trigger_custom_event` interface.\n\nThe `e` argument is an event instance that may become the subclass of the `Event` class, such as the `MouseEvent` or `TimerEvent`\\.\n\nThe following example rotates the rectangle when you click it. If the rectangle rotated 90 degrees, then the custom event (`rotate_90_degrees`) is triggered, and the `on_rotate_90_degrees` handler (custom event) is called and display the second rectangle (toggle the `visible` property):':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n# Custom event type name.\nROTATE_90_DEGREES: str = \'rotate_90_degrees\'\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_rectangle_click(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.unbind_click(on_rectangle_click)\n    options_: _RectOptions = {\'rectangle\': e.this}\n    timer: ap.Timer = ap.Timer(\n        on_timer, delay=ap.FPS.FPS_60, repeat_count=90,\n        options=options_)\n    timer.timer_complete(\n        on_timer_complete,\n        options=options_)\n    timer.start()\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    rectangle.rotation_around_center += 1\n\n\ndef on_timer_complete(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that timer calls when its end.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    rectangle.trigger_custom_event(custom_event_type=ROTATE_90_DEGREES)\n\n\ndef on_rotate_90_degrees(e: ap.Event, options: _RectOptions) -> None:\n    """\n    The handler that the rectangle rates 90 degrees (custom event).\n\n    Parameters\n    ----------\n    e : ap.Event\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    rectangle.visible = ap.Boolean(True)\n\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_1.click(on_rectangle_click)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50)\nrectangle_2.visible = ap.Boolean(False)\n\ne: ap.Event = ap.Event(this=rectangle_1)\nrectangle_1.bind_custom_event(\n    custom_event_type=ROTATE_90_DEGREES, handler=on_rotate_90_degrees, e=e,\n    options={\'rectangle\': rectangle_2})\n\nap.save_overall_html(\n    dest_dir_path=\'bind_and_trigger_custom_event_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/bind_and_trigger_custom_event_basic_usage/index.html" width="250" height="150"></iframe>':  # noqa
    '',

    '## bind_custom_event API':
    '',

    '<!-- Docstring: apysc._event.custom_event_interface.CustomEventInterface.bind_custom_event -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `bind_custom_event(self, custom_event_type:Union[apysc._event.custom_event_type.CustomEventType, str], handler:Callable[[Any, Any], NoneType], e:apysc._event.event.Event, *, options:Union[Any, NoneType]=None, in_handler_head_expression:str=\'\') -> str`<hr>\n\n**[Interface summary]** Add a custom event listener setting.<hr>\n\n**[Parameters]**\n\n- `custom_event_type`: CustomEventType or str\n  - Target custom event type.\n- `handler`: _Handler\n  - Callable that this instance calls when its event\'s dispatching.\n- `e`: Event\n  - Event instance.\n- `options`: dict or None, default None\n  - Optional arguments dictionary to be passed to a handler.\n- `in_handler_head_expression`: str, default \'\'\n  - Optional expression to be added at the handler function\'s head position.\n\n<hr>\n\n**[Returns]**\n\n- `name`: str\n  - Handler\'s name.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_custom_event(\n...         e: ap.Event[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> e: ap.Event = ap.Event(this=rectangle)\n>>> _ = rectangle.bind_custom_event(\n...     custom_event_type=\'my_custom_event\',\n...     handler=on_custom_event, e=e)\n>>> # Do something here and then trigger the custom event\n>>> rectangle.trigger_custom_event(\n...     custom_event_type=\'my_custom_event\')\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [About the handler options\' type document](https://simon-ritchie.github.io/apysc/about_handler_options_type.html)':  # noqa
    '',

    '## trigger_custom_event API':
    '',

    '<!-- Docstring: apysc._event.custom_event_interface.CustomEventInterface.trigger_custom_event -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `trigger_custom_event(self, custom_event_type:Union[apysc._event.custom_event_type.CustomEventType, str]) -> None`<hr>\n\n**[Interface summary]** Add a custom event trigger setting.<hr>\n\n**[Parameters]**\n\n- `custom_event_type`: CustomEventType or str\n  - Target custom event type.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_custom_event(\n...         e: ap.Event[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> e: ap.Event = ap.Event(this=rectangle)\n>>> _ = rectangle.bind_custom_event(\n...     custom_event_type=\'my_custom_event\',\n...     handler=on_custom_event, e=e)\n>>> # Do something here and then trigger the custom event\n>>> rectangle.trigger_custom_event(\n...     custom_event_type=\'my_custom_event\')\n```':  # noqa
    '',

}
